/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#if defined ( XBNBT_MYSQL ) || defined ( BNBT_MYSQL )

 #include "bnbt.h"
 #include "bnbt_mysql.h"
 #include "util.h"
 
 #if defined ( XBNBT_MYSQL )
  #include "atom.h"
  #include "config.h"
 #endif

#if defined ( BNBT_MYSQL )

MYSQL *gpMySQL = 0;
string gstrMySQLHost = string( );
string gstrMySQLDatabase = string( );
string gstrMySQLUser = string( );
string gstrMySQLPassword = string( );
string gstrMySQLPrefix = string( );
unsigned int guiMySQLPort = 0;

const string UTIL_StringToMySQL( const string &cstrString )
{
	char *szMySQL = new char[cstrString.size( ) * 2 + 1];

	if( gpMySQL )
		mysql_real_escape_string( gpMySQL, szMySQL, cstrString.c_str( ), ( unsigned long )cstrString.size( ) );

	const string cstrMySQL( szMySQL );

	delete [] szMySQL;

	return cstrMySQL;
}

void UTIL_MySQLCreateDatabase( )
{
		string strCreate = "CREATE DATABASE IF NOT EXISTS " + gstrMySQLDatabase ;

		mysql_real_query( gpMySQL, strCreate.c_str( ), ( unsigned long )strCreate.size( ) );
		mysql_store_result( gpMySQL );

		if( mysql_errno( gpMySQL ) )
			UTIL_LogPrint( "mysql error - %s\n", mysql_error( gpMySQL ) );
}

void UTIL_MySQLCreateTables( )
{
	string strCreate[8];

	strCreate[0] = "CREATE TABLE IF NOT EXISTS torrents ( bhash BLOB, bseeders INT UNSIGNED NOT NULL, bleechers INT UNSIGNED NOT NULL, bcompleted INT UNSIGNED NOT NULL, badded DATETIME NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[1] = "CREATE TABLE IF NOT EXISTS allowed ( bhash BLOB, bname VARCHAR(255) NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[2] = "CREATE TABLE IF NOT EXISTS allowed_ex ( bhash BLOB, badded DATETIME NOT NULL, bsize BIGINT UNSIGNED NOT NULL, bfiles INT UNSIGNED NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[3] = "CREATE TABLE IF NOT EXISTS hashes ( bhash BLOB, PRIMARY KEY( bhash(20) ) )";
	strCreate[4] = "CREATE TABLE IF NOT EXISTS seeders ( bhash BLOB, bseeders INT UNSIGNED NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[5] = "CREATE TABLE IF NOT EXISTS leechers ( bhash BLOB, bleechers INT UNSIGNED NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[6] = "CREATE TABLE IF NOT EXISTS completed ( bhash BLOB, bcompleted INT UNSIGNED NOT NULL, PRIMARY KEY( bhash(20) ) )";
	strCreate[7] = "CREATE TABLE IF NOT EXISTS dstate ( bhash BLOB, bid BLOB, bip VARCHAR(15) NOT NULL, bport SMALLINT UNSIGNED NOT NULL, bkey BLOB, buploaded BIGINT UNSIGNED NOT NULL, bdownloaded BIGINT UNSIGNED NOT NULL, bleft BIGINT UNSIGNED NOT NULL, btime DATETIME NOT NULL, PRIMARY KEY( bhash(20), bid(20) ) )";

	unsigned char ucQueries = 0;

	if( CFG_GetInt( "mysql_override_dstate", 0 ) == 0 ? false : true )
		ucQueries = sizeof( strCreate ) / sizeof( string );

	for( unsigned char ucLoop = 0; ucLoop <= ucQueries; ucLoop++ )
	{
		mysql_real_query( gpMySQL, strCreate[ucLoop].c_str( ), ( unsigned long )strCreate[ucLoop].size( ) );

		mysql_store_result( gpMySQL );

		if( mysql_errno( gpMySQL ) )
			UTIL_LogPrint( "mysql error - %s\n", mysql_error( gpMySQL ) );
	}
}

CMySQLQuery :: CMySQLQuery( const string cstrQuery )
{
	mysql_real_query( gpMySQL, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

	m_pRes = mysql_store_result( gpMySQL );

	if( mysql_errno( gpMySQL ) )
	{
		m_pRes = 0;

		if( gbDebug )
			UTIL_LogPrint( "mysql error - %s\n", mysql_error( gpMySQL ) );
	}
}

CMySQLQuery :: ~CMySQLQuery( )
{
	if( m_pRes )
		mysql_free_result( m_pRes );
}

const vector<string> CMySQLQuery :: nextRow( )
{
	vector<string> vecReturn;

	if( m_pRes )
	{
		const unsigned int num_fields( mysql_num_fields( m_pRes ) );

		vecReturn.reserve( num_fields );

		const MYSQL_ROW row( mysql_fetch_row( m_pRes ) );

		if( row != 0 )
		{
			const unsigned long *lengths( mysql_fetch_lengths( m_pRes ) );

			for( unsigned long ulCount = 0; ulCount < num_fields; ulCount++ )
				vecReturn.push_back( string( row[ulCount], lengths[ulCount] ) );
		}
	}

	return vecReturn;
}
#endif

/***
*
* XBNBT Beta 81b.3.5 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#if defined ( XBNBT_MYSQL )

MYSQL *gpMySQLUsers = 0;
string gstrMySQLUsersHost = string( );
string gstrMySQLUsersDatabase = string( );
string gstrMySQLUsersUser = string( );
string gstrMySQLUsersPassword = string( );
string gstrMySQLUsersTable = string( );
string gstrMySQLUsersAdmin = string( );
unsigned int guiMySQLUsersPort = 0;
bool gbMySQLUsersOverrideUsers = false;
unsigned char gucMySQLUsersMemberAccess = 0;
string gstrMySQLUsersName = string( );

const bool UTIL_StartMySQLUsers( )
{
	bool bResult = false;
	unsigned char ucCount = 0;

	MILLISLEEP( 100 );

	while( !bResult && ( ucCount < 5 ) )
	{
		bResult = UTIL_OpenMySQLUsers( );

		ucCount++;

		MILLISLEEP( 1000 );
	}

	if( bResult )
	{
		if( UTIL_TestMySQLUsersColumn( ) )
			bResult = UTIL_SetMySQLUsersMemberDefaultAccess( );
		else
			bResult = UTIL_CreateMySQLUsersColumn( );

		if( bResult )
			bResult = UTIL_SetMySQLUsersMemberAccess( gstrMySQLUsersAdmin, 127 );
	}

	return bResult;
}

/*
const string UTIL_StringToMySQLUsers( const string &cstrString )
{
	char *szMySQLUsers = new char[cstrString.size( ) * 2 + 1];

	if( gpMySQLUsers )
		mysql_real_escape_string( gpMySQLUsers, szMySQLUsers, cstrString.c_str( ), ( unsigned long )cstrString.size( ) );

	const string cstrMySQLUsers( szMySQLUsers );

	delete [] szMySQLUsers;

	return cstrMySQLUsers;
}
*/

const bool UTIL_OpenMySQLUsers(void)
{
	// Start MySQL Users mysql
	gpMySQLUsers = mysql_init( 0 );

	if( !gpMySQLUsers )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["init_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		
		return false;
	}
	
	if( !( mysql_real_connect( gpMySQLUsers, gstrMySQLUsersHost.c_str( ), gstrMySQLUsersUser.c_str( ), gstrMySQLUsersPassword.c_str( ), gstrMySQLUsersDatabase.c_str( ), guiMySQLUsersPort, 0, 0 ) ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["connect_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["success_mysql_users"] + "\n" ).c_str( ) );
	
	return true;
}

const bool UTIL_TestMySQLUsersColumn(void)
{
	const string cstrQuery( "SELECT xbnbt_perm FROM " + gstrMySQLUsersTable + " LIMIT 1" );

	mysql_real_query( gpMySQLUsers, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

	mysql_store_result( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["no_mysql_users_column"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["found_mysql_users_column"] + "\n" ).c_str( ) );
	
	return true;
}

const bool UTIL_SetMySQLUsersMemberDefaultAccess(void)
{
	const string cstrQuery( "ALTER TABLE " + gstrMySQLUsersTable + " ALTER COLUMN xbnbt_perm SET DEFAULT \'" + CAtomInt( gucMySQLUsersMemberAccess ).toString( ) + "\'" );
	
	mysql_real_query( gpMySQLUsers, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

	mysql_store_result( gpMySQLUsers );
	
	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_set_default_access"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["set_default_access"] + "\n" ).c_str( ) );
	
	return true;
}

const bool UTIL_SetMySQLUsersMemberAccess( const string &cstrMySQLUsersMember, const unsigned char &cucMySQLUsersAccess )
{
	const string cstrQuery( "UPDATE " + gstrMySQLUsersTable + " SET xbnbt_perm=\'" + CAtomInt( cucMySQLUsersAccess ).toString( ) + "\' WHERE " + gstrMySQLUsersName + "=\'" + cstrMySQLUsersMember + "\'" );

	mysql_real_query( gpMySQLUsers, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

	mysql_store_result( gpMySQLUsers );
	
	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_set_member_access"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["set_member_access"] + "\n" ).c_str( ) );

	return true;
}

const bool UTIL_CreateMySQLUsersColumn(void)
{
	const string cstrQuery( "ALTER TABLE " + gstrMySQLUsersTable + " ADD COLUMN xbnbt_perm SMALLINT DEFAULT \'" + CAtomInt( gucMySQLUsersMemberAccess ).toString( ) + "\'" );

	mysql_real_query( gpMySQLUsers, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

	mysql_store_result( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_created_access_column"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );

		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["created_access_column"] + "\n" ).c_str( ) );
	return true;
}

const bool UTIL_CloseMySQLUsers(void)
{
	mysql_close( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_closed_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );

		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["closed_mysql_users"] + "\n" ).c_str( ) );

	return true;
}

CMySQLUsersQuery :: CMySQLUsersQuery( const string cstrQuery )
{
	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["mysql_users_query"] + "\n" ).c_str( ), cstrQuery.c_str( ) );

	if( !mysql_ping( gpMySQLUsers ) )
	{
		mysql_real_query( gpMySQLUsers, cstrQuery.c_str( ), ( unsigned long )cstrQuery.size( ) );

		m_pRes = mysql_store_result( gpMySQLUsers );

		if( mysql_errno( gpMySQLUsers ) )
		{
			m_pRes = 0;

			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["mysql_users_query_error"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		}
	}
	else
	{
		if( mysql_errno( gpMySQLUsers ) )
		{
			m_pRes = 0;
			
			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["ping_error"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		}
	}
}

CMySQLUsersQuery :: ~CMySQLUsersQuery( )
{
	if( m_pRes )
		mysql_free_result( m_pRes );
}

const vector<string> CMySQLUsersQuery :: nextRow( )
{
	vector<string> vecReturn;

	if( m_pRes )
	{
		const unsigned int num_fields( mysql_num_fields( m_pRes ) );

		vecReturn.reserve( num_fields );

		const MYSQL_ROW row( mysql_fetch_row( m_pRes ) );

		if( row )
		{
			const unsigned long *lengths( mysql_fetch_lengths( m_pRes ) );

			for( unsigned long ulCount = 0; ulCount < num_fields; ulCount++ )
				vecReturn.push_back( string( row[ulCount], lengths[ulCount] ) );
		}
	}

	return vecReturn;
}

#endif

#endif
